# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-29 09:10
from __future__ import unicode_literals

import localized_fields.fields.text_field
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0013_auto_20190418_0733")]

    operations = [
        migrations.AddField(
            model_name="question",
            name="static_content",
            field=localized_fields.fields.text_field.LocalizedTextField(
                blank=True, null=True, required=[]
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="type",
            field=models.CharField(
                choices=[
                    ("multiple_choice", "multiple_choice"),
                    ("integer", "integer"),
                    ("float", "float"),
                    ("date", "date"),
                    ("choice", "choice"),
                    ("textarea", "textarea"),
                    ("text", "text"),
                    ("table", "table"),
                    ("form", "form"),
                    ("file", "file"),
                    ("dynamic_choice", "dynamic_choice"),
                    ("dynamic_multiple_choice", "dynamic_multiple_choice"),
                    ("static", "static"),
                ],
                max_length=23,
            ),
        ),
    ]
