# Generated by Django 2.2.6 on 2019-10-31 08:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0025_dynamicoption_historicaldynamicoption")]

    operations = [
        migrations.RemoveField(model_name="dynamicoption", name="value"),
        migrations.RemoveField(model_name="historicaldynamicoption", name="value"),
        migrations.AddField(
            model_name="dynamicoption",
            name="slug",
            field=models.CharField(default="unset", max_length=255),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicaldynamicoption",
            name="slug",
            field=models.CharField(default="unset", max_length=255),
            preserve_default=False,
        ),
    ]
