# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-02-13 10:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_workflow", "0009_auto_20190213_0939")]

    operations = [
        migrations.AddField(
            model_name="task",
            name="lead_time",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="Time in seconds task may take to be processed.",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="workitem",
            name="deadline",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
