# Generated by Django 2.2.8 on 2020-01-13 07:27

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0026_auto_20191031_0834")]

    operations = [
        migrations.RemoveIndex(model_name="answer", name="form_answer_date_6485f0_idx"),
        migrations.RemoveIndex(model_name="answer", name="form_answer_meta_98edaa_gin"),
        migrations.RemoveIndex(
            model_name="document", name="form_docume_meta_333f0b_gin"
        ),
        migrations.RemoveIndex(model_name="form", name="form_form_meta_a2eed1_gin"),
        migrations.RemoveIndex(model_name="option", name="form_option_meta_c09f31_gin"),
        migrations.RemoveIndex(
            model_name="question", name="form_questi_meta_ea1ba0_gin"
        ),
        migrations.AddIndex(
            model_name="answer",
            index=models.Index(fields=["date"], name="caluma_form_date_c923c1_idx"),
        ),
        migrations.AddIndex(
            model_name="answer",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["meta", "value"], name="caluma_form_meta_32fc37_gin"
            ),
        ),
        migrations.AddIndex(
            model_name="document",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["meta"], name="caluma_form_meta_89b894_gin"
            ),
        ),
        migrations.AddIndex(
            model_name="form",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["meta"], name="caluma_form_meta_796a95_gin"
            ),
        ),
        migrations.AddIndex(
            model_name="option",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["meta"], name="caluma_form_meta_e0f506_gin"
            ),
        ),
        migrations.AddIndex(
            model_name="question",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["meta"], name="caluma_form_meta_e7f451_gin"
            ),
        ),
    ]
