# Generated by Django 2.2.3 on 2019-07-08 09:05

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "caluma_form",
            "0020_historicalanswer_historicalanswerdocument_historicaldocument_historicalfile_historicalform_historica",
        )
    ]

    operations = [
        migrations.AlterField(
            model_name="form",
            name="source",
            field=models.ForeignKey(
                blank=True,
                help_text="Reference this form has been copied from",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="caluma_form.Form",
            ),
        ),
        migrations.AlterField(
            model_name="option",
            name="source",
            field=models.ForeignKey(
                blank=True,
                help_text="Reference this option has been copied from",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="caluma_form.Option",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="row_form",
            field=models.ForeignKey(
                blank=True,
                help_text="Form that represents rows of a TableQuestion",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="caluma_form.Form",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="source",
            field=models.ForeignKey(
                blank=True,
                help_text="Reference this question has been copied from",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="caluma_form.Question",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="sub_form",
            field=models.ForeignKey(
                blank=True,
                help_text="Form referenced in a FormQuestion",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="caluma_form.Form",
            ),
        ),
    ]
