# Generated by Django 2.2.3 on 2019-07-29 14:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0022_add_gin_index_to_jsonfields")]

    operations = [
        migrations.AddField(
            model_name="historicaloption",
            name="is_archived",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="option",
            name="is_archived",
            field=models.BooleanField(default=False),
        ),
    ]
